from typing import List, Union

from fastapi import HTTPException
from pydantic import BaseModel, Field, root_validator
from ..session import session, AccessorIdError


class PipelineParams(BaseModel):
    keep_interm: bool = Field(False, description='Keep accessors to intermediate images in session')

    @root_validator(pre=False)
    def models_are_loaded(cls, dd):
        for k, v in dd.items():
            if k.endswith('model_id') and v not in session.describe_loaded_models().keys():
                raise HTTPException(status_code=409, detail=f'Model with {k} = {v} has not been loaded')
        return dd

    @root_validator(pre=False)
    def accessors_are_loaded(cls, dd):
        for k, v in dd.items():
            if k.endswith('accessor_id'):
                try:
                    info = session.get_accessor_info(v)
                except AccessorIdError as e:
                    raise HTTPException(status_code=409, detail=str(e))
                if not info['loaded']:
                    raise HTTPException(status_code=409, detail=f'Accessor with {k} = {v} has not been loaded')
        return dd


class PipelineRecord(BaseModel):
    output_accessor_id: str
    interm_accessor_ids: Union[List[str], None]
    success: bool
    timer: dict
